/**
 * main.c for a simple test projct of FreeRTOS
 * Learned from
 * https://iotexpert.com/2017/02/23/psoc4-freertos-port/
 */
#include "project.h"
#include <stdio.h>
#include "FreeRTOS.h"
#include "task.h"
#include "setupFreeRTOS.h"
#include "FreeRTOSConfig.h"

char str[128] ; /* print buffer */

void init_hardware(void)
{
    CyGlobalIntEnable; /* Enable global interrupts. */
    UART_Start() ;
    CyBle_Init() ;
}

void LED_Task(void *arg)
{
    (void) arg ;

    UART_UartPutString("LED Task Started!\n") ;
    UART_UartPutString(str) ;
    
    while(1) {
        LED_R_Write(~LED_R_Read()) ;
        vTaskDelay(500) ;
    }
}

void UART_Task(void *arg)
{
    (void) arg ;
    unsigned int loop = 0 ;
    
    UART_UartPutString("UART Task Started!\n"); 
    UART_UartPutString(str) ;
    
    while(1) {
        sprintf(str, "Loop %d\n", loop++) ;
        UART_UartPutString(str) ;
        vTaskDelay(1200) ;
    }   
}

int main(void)
{
    init_hardware() ;
    
    sprintf(str, "FreeRTOS test program (%s %s)\n", __DATE__, __TIME__) ;
    UART_UartPutString(str) ;
    
    setupFreeRTOS() ;
    UART_UartPutString("setupFreeRTOS() done\n") ;
    
    UART_UartPutString("Creating the first task ... ") ;
    /* Create LED task, which will control the intensity of the LEDs */
    xTaskCreate(
        LED_Task,       /* Task function */
        "LED Blink",    /* Task name (string) */
        200,            /* Task stack, allocated from heap */
        0,              /* No param passed to task function */
        1,              /* Low priority */
        0               /* Not using tha task handle */
    ) ;
    UART_UartPutString("done\n") ;
    
    UART_UartPutString("Creating the second task ... ") ;
    xTaskCreate(
        UART_Task,
        "UART Task",
        200,
        0,
        1,
        0
    ) ;
    UART_UartPutString("done\n") ;
        
    
    UART_UartPutString("Starting vTaskStartScheduler()\n") ;

    vTaskStartScheduler() ;

    while(1) {
        /* We should not arrive here, but just in case ;-) */
    }
}

